"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var get_response_examples_1 = __importDefault(require("./get-response-examples"));
/**
 * With an OpenAPI Operation Object return back a collection of examples for any callbacks that may
 * be present.
 *
 * @param operation Operation to retrieve callback examples from.
 */
function getCallbackExamples(operation) {
    var ret = [];
    // spreads the contents of the map for each callback so there's not nested arrays returned
    return ret.concat.apply(ret, Object.keys(operation.callbacks || {}).map(function (identifier) {
        var callback = operation.callbacks[identifier];
        // spreads the contents again so there's not nested arrays returned
        return []
            .concat.apply([], Object.keys(callback).map(function (expression) {
            return Object.keys(callback[expression]).map(function (method) {
                var pathItem = callback[expression];
                var example = (0, get_response_examples_1["default"])(pathItem[method]);
                if (example.length === 0)
                    return false;
                return {
                    identifier: identifier,
                    expression: expression,
                    method: method,
                    example: example
                };
            });
        })).filter(Boolean);
    }));
}
exports["default"] = getCallbackExamples;
