const serialize = require('serialize-javascript');
const AWS = require('aws-sdk');
const request = require('request')
var urlExists = require('url-exists');
const { getFrame, createGifFromVideo, trimVideo } = require('../library/video-lib')

const db = require('../models/bootstrap')
const { videoRenderEmail, testEmail } = require("../services/emails/Email")

const { inserSingleRow, insertMultiRows } = require("../services/google/sheet-api")

const Op = db.Sequelize.Op;
const Template = db.template;
const Campaign = db.campaign;
const Landingpage = db.landingpage;
const Personalize = db.personalization;
const User = db.user;


const fs = require('fs')
const axios = require('axios').default;

var cron = require('node-cron');

cron.schedule('* * * * *', async () => {
    // console.log("hii"); 
    const { count } = await Campaign.findAndCountAll({ 
        where: { 
            render_progress: {
                [Op.gt]: 0 ,
                [Op.lt]: 100 ,
            },
            process_flow: 2
        },
        // logging: console.log,
        attributes: ['id', 'campaign_data', 'user_id', 'audio_path'],
    });

    console.log("processing video="+count);
    
    if(count >= 5){
        console.log('Please wait...');
    }else{
        const {count,rows} = await Campaign.findAndCountAll({
            limit: 3,
            where: { 
                render_progress: 0,
                process_flow: 2
            },
            // logging: console.log,
            attributes: ['id', 'campaign_data', 'user_id', 'audio_path'],
            order: [['id', 'ASC']]
        });

        if(count > 0){

            let n = rows.length;
            console.log("n="+n);

            rows.slice(0,4).map(ele => {
                const { id, campaign_data,user_id,audio_path } = ele;
                console.log("rendering campaign="+id+" Nexrender 1");
                var scene_data = JSON.parse(campaign_data);
                makeRenderRequest(id, user_id, scene_data, audio_path,1);
            });
           
        }else{
            console.log("Not any campaign to render");
        }
    }
});

checkCampaign = async (req, res) => {
    try {
        const name = req.body.name
        const check_row = await Campaign.findAll({ where: { name, user_id: req._user_id } })
        if (check_row.length > 0) {
            res.send({ status: 1 })
        }
        else {
            res.send({ status: 0 })

        }
    } catch (error) {
        res.send({ message: error.message })

    }

}


insert = async (req, res) => {
    try {
        // res.send({ msg: "Hello" })
        const { template_id, campaign_name } = req.body

        const data = await Template.findOne({
            attributes: ['slide_data'],
            where: {
                main_id: template_id
            },
        })





        const name = campaign_name.toLowerCase();
        const check_row = await Campaign.findAll({ where: { name, user_id: req._user_id } })
        if (check_row.length > 0) return res.status(403).send({ error: "error", message: "Campaign Already Present" });


        const insertData = await Campaign.create({
            name, template_id, user_id: req._user_id,
            campaign_data: data.slide_data
        })


        if (insertData.id) {
            await appendLandingPage(insertData.id)
        }

        res.send({ id: insertData.id })

    } catch (error) {
        res.status(404).send({ message: error.message })
    }
}


insertCampaign = async (req, res) => {
    try {
        const { addTags, campaign_name, addDesc, filename, video_type } = req.body

        // console.log("keyword=", JSON.stringify(addTags.toString()));
        const name = campaign_name.toLowerCase();
        const check_row = await Campaign.findAll({ where: { name, user_id: req._user_id } })

        if (check_row.length > 0) return res.status(403).send({ error: "error", message: "Campaign Already Present" });


        const insertData = await Campaign.create({
            name,
            user_id: req._user_id,
            keywords: addTags.toString(),
            description: addDesc,
            video_type,
            video_path: filename,
            process_flow: 3,

        })

        if (insertData.id) {
            await appendLandingPage(insertData.id)

            //create gif from video
            if (video_type === 'cust_upload') {

                // const frame = await getFrame(filename, insertData.id, '00:00:03', `/${req._user_id}/campaigns/${insertData.id}/`)
                const frame = await getFrame_lambda(filename, insertData.id, '00:00:03', `/${req._user_id}/campaigns/${insertData.id}/`, req._user_id)


                if (!frame) res.status(402).send({ body: { message: 'Something went wrong' } })

                await createGif(filename, insertData.id, req._user_id)

                await Campaign
                    .update(
                        { poster: frame.url, default_poster: frame.url }
                        , {
                            where: { id: insertData.id }
                        }
                    )
            }
        }
        // console.log(insertData);

        res.send({ id: insertData.id })

    } catch (error) {
        res.status(404).send({ message: error.message })
    }
}


appendLandingPage = async (camp_id) => {


    try {

        // insert Default landing page 1
        const content = `<div class="landing_page_1"> <header class="header"><div class="container"><div class="row"><div class="col-xl-7 col-lg-8 col-md-12 m-auto text-center"><div class="logo"><a href="#"><img src="https://d37q3r06begyqi.cloudfront.net/VidAmaze/app-images/logo.png" class="img-fluid" alt="img"></a></div><h1 class="mt-3">Ready to Increase Your Affiliate Sales?</h1><p class="m-0 mb-3">Learn how to apply our proven Affiliate Marketing System to increase your passive income stream and maximize affiliate sales in just 2 hours a week.</p><div class="video_ld1 mceNonEditable ms-4 me-4 position-relative mb-4"><a href="#" class="play_v"><img src="https://d37q3r06begyqi.cloudfront.net/VidAmaze/app-images/v_play1.png" class="img-fluid" alt="img"></a><img src="https://engageleads-gallery.s3.us-west-2.amazonaws.com/VidAmaze/app-images/video_preview_back_1.png" class="w-100" alt="img"></div><a href="#" class="btn shadow-none">Buy Now <i class="fa fa-long-arrow-right ms-2"></i></a></div></div></div></header><main class="midwrap"><div class="container"><div class="row"><div class="col-xl-7 col-lg-8 col-md-12 m-auto text-center"><h2>Let's grow your business together.</h2><p class="ms-5 me-5 pe-5 ps-5">Learn how to apply our proven Affiliate Marketing System to increase your passive income stream and maximize affiliate sales in just 2 hours a week.</p><a href="#" class="btn btn2 shadow-none">Sign Up Now</a></div></div></div></main> <footer class="footer"><div class="container"><div class="row"><div class="col-md-3"><a href="#" class="ld1_logo2 d-inline-block mt-4"><img src="https://d37q3r06begyqi.cloudfront.net/VidAmaze/app-images/logo.png" class="img-fluid" alt="img"></a></div><div class="col-md-9"><div class="footer_right"><div class="ft_rw1 d-flex justify-content-between align-items-center p-4"><div class="copy_right">©2022 Vidamaze. All rights reserved.</div><div class="social"><a href="#"><i class="fa fa-facebook-f"></i></a><a href="#"><i class="fa fa-twitter"></i></a><a href="#"><i class="fa fa-pinterest-p"></i></a></div></div><hr class="m-0"><div class="ft_rw2 p-4 pt-3 pb-3"><a href="#">Privacy Policy</a> </div></div></div></div></div></footer> </div>`;
        const insertData = await Landingpage.create({
            camp_id, content: content, draft_content: content, landing_page_id: 1
        })


    } catch (error) {
        console.log(error)
    }

}

//create Gif 
createGif = async (filename, camp_id, user_id) => {
    try {
        console.log("Generating Gif",filename);
        const check_row = await Campaign.findOne({ where: { id: camp_id, user_id } })

        if (!check_row) console.log('Something went wrong')

        const gif = await axios.post(process.env.vidamaze_AWS_LAMDA_GIF, {
            url:filename,
            start:0,
            end:5,
            camp_id,
            upload_dir_path:`/${user_id}/campaigns/${camp_id}/gif/`
        });

        // const gif = await createGifFromVideo(filename, camp_id, `/${user_id}/campaigns/${camp_id}/gif/`,0,4)
        console.log(gif);
        console.log(gif.data);

        if (!gif.data.url) console.log('Something went wrong')

        await Campaign
            .update(
                {
                    gif_url: gif.data.url
                },
                {
                    where: { id: camp_id }
                }
            )

    } catch (error) {
        console.log(error)
    }
}


//create frame 
getFrame_lambda = async (filename, camp_id, duration, upload_dir_path, user_id) => {
    try {
        console.log("Generating Frame",filename);
        const check_row = await Campaign.findOne({ where: { id: camp_id, user_id } })

        if (!check_row) console.log('Something went wrong')

        const frame = await axios.post(process.env.vidamaze_AWS_LAMDA_GIF, {
            url:filename,
            camp_id,
            duration,
            upload_dir_path
        });

        // const gif = await createGifFromVideo(filename, camp_id, `/${user_id}/campaigns/${camp_id}/gif/`,0,4)

        if (!frame.data.url) console.log('Something went wrong')

        // console.log(frame.data)
        return frame.data;

    } catch (error) {
        console.log(error)
    }
}



fetchCampaign = async (req, res) => {

    try {
        const camp_id = req.params.camp_id;
        var wherecond;
        // console.log(camp_id,req._user_id)
        if (req._user_id === 1) {
            wherecond = { id: camp_id };
        }
        else {
            wherecond = { user_id: req._user_id, id: camp_id };

        }
        // res.send("Hello")
        const data = await Campaign.findOne({
            where: wherecond,
            // logging: console.log
        })

        // console.log(data);
        res.send({ body: data })
    } catch (error) {

        res.send(error)
    }
}


fetchSingleCampaign = async (req, res) => {
    try {
        var id = req.params.id;
        const data = await Campaign.findOne({
            where: {
                id,
                user_id: req._user_id
            }
        });

        if (!data) {
            return res.send({ status: false })
        }
        res.send(data)
    } catch (error) {
        res.send({ message: error })
    }
}
// upload image to s3

uploadImage = async (req, res) => {
    try {
        const file = req.file;
        let uploadPath = `${process.env.vidamaze_S3_DIR_NAME}/users/${req._user_id}/images/${file.originalname}`

        if (req.body.camp_id) {
            uploadPath = `${process.env.vidamaze_S3_DIR_NAME}/users/${req._user_id}/slide-images/${file.originalname}`
        }
        if (req.file == undefined) {
            return res.status(400).send({ message: "Please upload a file!" });
        }

        console.log(req.file)

        let s3bucket = new AWS.S3({
            accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
            secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
            region: process.env.vidamaze_S3_REGION
        });

        var params = {
            Bucket: process.env.vidamaze_S3_BUCKET_NAME,
            Key: uploadPath,
            Body: file.buffer,
            ContentType: file.mimetype,
            ACL: "public-read"
        };



        s3bucket.upload(params, function (err, data) {
            if (err) {
                res.status(500).send({ error: true, Message: err });
            } else {
                res.status(200).send({ url: process.env.vidamaze_S3_SERVER_LINK + data.Key });
            }
        });


    }
    catch (err) {

        console.log(err);

        if (err.code == "LIMIT_FILE_SIZE") {
            return res.status(500).send({
                message: "File size cannot be larger than 2MB!",
            });
        }

        res.status(500).send({
            message: `Could not upload the file: ${req.file.originalname}. ${err}`,
        });
    }
}

uploadVideo = async (req, res) => {
    try {
        const file = req.file;

        if (req.file == undefined) {
            return res.status(400).send({ message: "Please upload a file!" });
        }

        let s3bucket = new AWS.S3({
            accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
            secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
            region: process.env.vidamaze_S3_REGION
        });

        var params = {
            Bucket: process.env.vidamaze_S3_BUCKET_NAME,
            Key: `${process.env.vidamaze_S3_DIR_NAME}/users/${req._user_id}/uploads/${file.originalname}`,
            Body: file.buffer,
            ContentType: file.mimetype,
            ACL: "public-read"
        };

        s3bucket.upload(params, async function (err, data) {
            if (err) {
                // console.log(err)
                return res.status(500).send({ error: true, Message: err });
            } else {
                // console.log(data.Location)
                // console.log(data.Key)
                return res.status(200).send({ url: process.env.vidamaze_S3_SERVER_LINK + data.Key });
            }
        });


    }
    catch (err) {


        if (err.code == "LIMIT_FILE_SIZE") {
            return res.status(500).send({
                message: "File size cannot be larger than 15MB!",
            });
        }

        res.status(500).send({
            message: `Could not upload the file: ${req.file.originalname}. ${err}`,
        });
    }
}

uploadAudio = async (req, res) => {
    try {
        const file = req.file;

        if (req.file == undefined) {
            return res.status(400).send({ message: "Please upload a file!" });
        }

        let s3bucket = new AWS.S3({
            accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
            secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
            region: process.env.vidamaze_S3_REGION
        });

        var params = {
            Bucket: process.env.vidamaze_S3_BUCKET_NAME,
            Key: `${process.env.vidamaze_S3_DIR_NAME}/users/${req._user_id}/uploads/audio/${file.originalname}`,
            Body: file.buffer,
            ContentType: file.mimetype,
            ACL: "public-read"
        };

        s3bucket.upload(params, async function (err, data) {
            if (err) {
                // console.log(err)
                return res.status(500).send({ error: true, Message: err });
            } else {
                // console.log(data.Location)
                // console.log(data.Key)
                return res.status(200).send({ url: process.env.vidamaze_S3_SERVER_LINK + data.Key });
            }
        });


    }
    catch (err) {


        if (err.code == "LIMIT_FILE_SIZE") {
            return res.status(500).send({
                message: "File size cannot be larger than 15MB!",
            });
        }

        res.status(500).send({
            message: `Could not upload the file: ${req.file.originalname}. ${err}`,
        });
    }
}

// Fetch Authenticated Users images

fetchImages = (req, res) => {
    try {
        let s3 = new AWS.S3({
            accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
            secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
            region: process.env.vidamaze_S3_REGION
        });

        var getParams = {
            Bucket: process.env.vidamaze_S3_BUCKET_NAME,
            Prefix: process.env.vidamaze_S3_DIR_NAME + '/users/' + req._user_id + '/images'
        }

        s3.listObjectsV2(getParams, function (err, data) {

            // Handle any error and exit

            if (err)
                return res.status(404).send(err);

            if (data.Contents && data.Contents.length < 1)
                return res.status(404).send('Not Found')

            let newData = [];

            newData = data.Contents.map(obj => {
                let new_obj = { ...obj }
                // new_obj.Key = process.env.vidamaze_S3_SERVER_LINK + obj.Key
                new_obj.Key = "https://engageleads-gallery.s3.us-west-2.amazonaws.com/" + obj.Key


                return new_obj
            })

            res.send(newData)
        });
    } catch (error) {

    }
}


fetchMedia = (req, res) => {
    try {
        let s3 = new AWS.S3({
            accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
            secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
            region: process.env.vidamaze_S3_REGION
        });

        var getParams = {
            Bucket: process.env.vidamaze_S3_BUCKET_NAME,
            Prefix: process.env.vidamaze_S3_DIR_NAME + '/users/' + req._user_id + '/uploads'
        }

        s3.listObjectsV2(getParams, function (err, data) {

            // Handle any error and exit

            if (err)
                return res.status(404).send(err);

            if (data.Contents && data.Contents.length < 1)
                return res.status(404).send('Not Found')

            let newData = [];

            newData = data.Contents.map(obj => {
                let new_obj = { ...obj }
                // new_obj.Key = process.env.vidamaze_S3_SERVER_LINK + obj.Key
                new_obj.Key = `https://engageleads-gallery.s3.us-west-2.amazonaws.com/` + obj.Key


                return new_obj
            })

            res.send(newData)
        });
    } catch (error) {

    }
}


/* Fetch Image and upload to S3 */

uploadURLImage = async (req, res) => {
    try {
        var img_url = req.body.url;


        if (img_url.includes('m.media-amazon.com')) {
            return res.send({ url: img_url })

        }


        urlExists(img_url, function (err1, exists) {
            if (!exists)
                return res.status(404).send({ message: "URL NOT FOUND" });

            let s3 = new AWS.S3({
                accessKeyId: process.env.vidamaze_S3_ACCESS_ID,
                secretAccessKey: process.env.vidamaze_S3_SECRET_KEY,
                region: process.env.vidamaze_S3_REGION
            });

            request({
                url: img_url,
                encoding: null
            }, function (err, res1, body) {

                // console.log(res1)
                if (err)
                    return res.status(404).send({ message: "URL NOT FOUND" });


                const key = `${process.env.vidamaze_S3_DIR_NAME}/users/${req._user_id}/${(Math.random() + 1).toString(36).substring(7)}.${img_url.split(/[#?]/)[0].split('.').pop().trim()}`
                var params = {
                    Bucket: process.env.vidamaze_S3_BUCKET_NAME,
                    Key: key,
                    Body: body,
                    ContentType: res1.headers['content-type'],
                    ContentLength: res1.headers['content-length'],
                    ACL: "public-read"
                };
                s3.putObject(params, (res2) => {
                    return res.send({ url: process.env.vidamaze_S3_SERVER_LINK + key })
                });


            })



        });





        // console.log(main)

    } catch (error) {
        console.log(error)
    }
}


/* Save the video data as 
    1 - Edit Mode
    2 - Process Mode(Cant Edit Again)
*/

saveVideoData = async (req, res) => {
    // console.log(req.body)
    // res.send("Hello")
    try {

        const { process_flow, camp_id, video_data, total_duration } = req.body;
        // console.log(process_flow)

        const response = await Campaign
            .update(
                { campaign_data: video_data, process_flow: 1,video_duration:total_duration }
                , {
                    where: { id: camp_id, user_id: req._user_id },
                    logging: console.log

                }

            )


        if (response[0] === 1) {
            res.send({ status: process_flow })
        }
        else {
            throw new Error("Cant Update Data");
        }


    } catch (error) {
        res.status(500).send({ message: error })
    }

}


/* Save the audio URL */
configureAudio = async (req, res) => {
    try {
        const { camp_id, selectedAudio } = req.body;

        // generateAssets(req)

        const response = await Campaign
            .update(
                { audio_path: selectedAudio, process_flow: 2 }
                , {
                    where: { id: camp_id, user_id: req._user_id }
                }
            )

        if (response[0] === 1) {
            res.send({ response: 'Done' })
        }
        else {
            throw new Error("Cant Update Data");
        }

    } catch (error) {

    }
}

/* Generate Assets Data */
generateAssets = async (req) => {

    // console.log(req.body)
    // return false;
    var global_assets = [];
    try {

        const camp_id = req.body.camp_id

        const data = await Campaign.findOne({
            where: {
                id: req.body.camp_id,
                user_id: req._user_id
            }
        });

        var scene_data = JSON.parse(data.campaign_data)


        makeRenderRequest(camp_id, req._user_id, scene_data, req.body.selectedAudio)


    } catch (error) {

    }


}

/* Generate Template 1(Sales Promo 7) assets */

generateOneTwoThree = async (assets_data, camp_id, user_id) => {

    // console.log(assets_data)
    // return false;
    try {
        var assets = [];


        const new_data = await Promise.all(assets_data.map(async slideData => {
            var text_assets = {}
            var color_assets = {}
            var fontSize_assets = {}
            var fontFamily_assets = {}

            if (slideData.layerName.indexOf('logo') === 0 || slideData.layerName.indexOf('bg_image') === 0) {
                // console.log('Calling Inside')
                text_assets = {
                    "type": "image",
                    "layerName": slideData.layerName,
                    "src": slideData.src
                }
            } else if (slideData.layerName.indexOf('bg_video') === 0) {

                if (slideData.trim_end) {
                    const frame = await trimVideo(slideData.src, slideData.trim_start, slideData.trim_end, `/users/${user_id}/campaigns/${camp_id}/`)
                    text_assets = {
                        "type": "video",
                        "layerName": slideData.layerName,
                        "src": frame.url
                    }

                    console.log(slideData.src, frame)
                } else {
                    text_assets = {
                        "type": "video",
                        "layerName": slideData.layerName,
                        "src": slideData.src.split("#")[0]
                    }
                }

                // text_assets = {
                //     "type": "video",
                //     "layerName": slideData.layerName,
                //     "src": slideData.src.split("#")[0]
                // }


            }
            else if (slideData.layerName.indexOf('bg_color') === 0) {

                text_assets = {
                    "type": "data",
                    "layerName": slideData.layerName,
                    "property": "Source Text",
                    "value": slideData.text.replace("#", "")
                }
            }
            else {

                if (slideData.text) {

                    text_assets = {
                        "type": "data",
                        "layerName": slideData.layerName,
                        "property": "Source Text",
                        "value": slideData.text.replace(/  /g, "\n")
                    }
                }






                if (slideData.fontColor) {
                    var colorData = slideData.fontColor.split(",")
                    color_assets = {
                        "type": "data",
                        "layerName": slideData.layerName,
                        "property": "Source Text.fillColor",
                        "value": [(colorData[0] / 255).toFixed(1), (colorData[1] / 255).toFixed(1), (colorData[2] / 255).toFixed(1)]
                    }
                }



                if (slideData.fontSize) {
                    fontSize_assets = {
                        "type": "data",
                        "layerName": slideData.layerName,
                        "property": "Source Text.fontSize",
                        "value": slideData.fontSize
                    }
                }

                if (slideData.fontFamily) {
                    fontFamily_assets = {
                        "type": "data",
                        "layerName": slideData.layerName,
                        "property": "Source Text.font",
                        "value": slideData.fontFamily.charAt(0).toUpperCase() + slideData.fontFamily.slice(1)
                    }
                }
            }

            if (text_assets.value || text_assets.src) {
                assets.push(text_assets)
            }

            if (color_assets.value) {
                assets.push(color_assets)
            }

            if (fontSize_assets.value) {
                assets.push(fontSize_assets)
            }

            if (fontFamily_assets.value) {
                assets.push(fontFamily_assets)
            }

            return true;
        }))
        // console.log(assets);
        // return false;
        return assets;
    } catch (error) {
        console.log(error)
    }
}

/*  Make video render */
makeRenderRequest = async (camp_id, user_id, assets_data, audio_path, instance_id) => {
    var Nexrender_REST_URL;
    if(instance_id && instance_id === 1){
        Nexrender_REST_URL = "http://vidamaze-nexrender-14feb-437823891.us-west-2.elb.amazonaws.com:2022/api/render";
    }else if(instance_id && instance_id === 4){
        // Nexrender_REST_URL = process.env.vidamaze_REST_URL_1;
        Nexrender_REST_URL = process.env.vidamaze_REST_URL;
    }else if(instance_id && instance_id === 3){
        // Nexrender_REST_URL = process.env.vidamaze_REST_URL_1;
        Nexrender_REST_URL = process.env.vidamaze_REST_URL;
    }else if(instance_id && instance_id === 2){
        // Nexrender_REST_URL = process.env.vidamaze_REST_URL_1;
        Nexrender_REST_URL = process.env.vidamaze_REST_URL;
    }else{
        Nexrender_REST_URL = process.env.vidamaze_REST_URL;
    }

    console.log("call = ",Nexrender_REST_URL,camp_id)

    await Campaign.update(
                        { instance_details: Nexrender_REST_URL}, 
                        {
                            where: { id: camp_id }
                        }
                    )
    
    request.post({
        // url: 'http://ec2-35-164-8-97.us-west-2.compute.amazonaws.com:1309/api/render',
        url: Nexrender_REST_URL,
        form: {
            assets_data: JSON.stringify(assets_data),
            camp_id: camp_id,
            user_id,
            audio_path
        }
    }, (err, res, body) => {
        // console.log(err)
    })


}

/* Update the render video details */

setrenderData = async (req, res) => {
    const { camp_id, params } = req.body
    // console.log(params)
    if (params.video_path) {

        // console.log(req.body);
        const user = await db.sequelize.query(`SELECT users.email,users.id FROM campaigns RIGHT JOIN users on campaigns.user_id = users.id Where campaigns.id = ` + camp_id + ` LIMIT 1;`, { type: db.Sequelize.QueryTypes.SELECT })
        // console.log()
        // const frame = await getFrame(params.video_path, camp_id, '00:00:03', `/users/${user[0].id}/campaigns/${camp_id}/`)
        const frame = await getFrame_lambda(params.video_path, camp_id, '00:00:03', `/users/${user[0].id}/campaigns/${camp_id}/`, user[0].id)
        // console.log(frame);
        await createGif(params.video_path, camp_id, user[0].id)

        await Campaign
            .update(
                { video_path: params.video_path, process_flow: 3, poster: frame.url, default_poster: frame.url }
                , {
                    where: { id: camp_id }
                }
            )



        if (user) {
            await videoRenderEmail(user[0].email, process.env.vidamaze_CLIENT_URL, frame.url)
            console.log('Email Sent')
        }


    }
    else if (params.render_status) {
        await Campaign
            .update(
                { render_status: params.render_status }
                , {
                    where: { id: camp_id }
                }
            )
    } else {
        await Campaign
            .update(
                { render_progress: params.render_progress }
                , {
                    where: { id: camp_id }
                }
            )
    }

    res.send("Hello")
}


checkTestEmail = async (req, res) => {
    try {


        testEmail('akash', 'akasht@onlightenmedia.com')


        res.send("Hello")

    } catch (error) {

    }
}


// Fetch Users Campaigns

getCampaigns = async (req, res) => {

    try {
        // console.log(req)
        let search = '';
        // let sort = 0;
        const page_no = req.params.page_no ? req.params.page_no : 0;
        const total_row = 9;

        // if (req.query.sort) {
        //     sort = req.query.sort;
        // }

        if (req.query.search) {
            search = req.query.search
        }

        let page = 0;
        var wherecond = {}

        if (!Number.isNaN(page_no) && page_no > 0) {
            page = page_no
        }


        if (search) {
            wherecond = {
                [Op.or]: [
                    {
                        name: {
                            [Op.like]: `%${search}%`
                        }
                    }]
            }
        }

        if (req._user_id !== 1) {
            wherecond.user_id = { [Op.eq]: req._user_id };
        }


        if (parseInt(req.query.sort) === 1) {
            wherecond.process_flow = { [Op.eq]: 3 }
        } else if (parseInt(req.query.sort) === 2) {
            wherecond.process_flow = { [Op.eq]: 2 }
        } else if (parseInt(req.query.sort) === 3) {
            wherecond.process_flow = { [Op.eq]: 0 }
        }


        // console.log(wherecond)

        const campaigns = await Campaign.findAndCountAll({
            limit: total_row,
            offset: page * total_row,
            where: wherecond,
            attributes: ['id', 'name', 'render_status', 'render_progress', 'status', 'video_path', 'campaign_data', 'render_message', 'createdAt','updatedAt', 'user_id', 'process_flow', 'video_type', 'poster'],

            order: [['createdAt', 'DESC']]
        });

        // console.log(campaigns.rows)

        res.send({
            content: campaigns.rows,
            totalPages: Math.ceil(campaigns.count / total_row)
        })
    } catch (error) {
        console.log(error)
    }

}


getAllCampaigns = async (req, res) => {
    try {
        if (req._user_id === 1) {
            wherecond = { id: { [Op.gt]: 1 } };
        }
        else {
            wherecond = { user_id: req._user_id };
        }
        const campaigns = await Campaign.findAll({
            where: wherecond,
            attributes: ['id', 'name'],
            order: [['createdAt', 'DESC']]
        });

        // console.log(campaigns.rows)

        res.send({
            body: campaigns,
        })
    } catch (error) {

    }
}

// Delete Campaign 
deleteCampaign = async (req, res) => {
    try {
        await Campaign.destroy({
            where: {
                id: req.params.id
            }
        })

        res.send({ status: 200, message: "Successfully deleted" })
    } catch (error) {

    }
}

//update campaign
updateCampaign = async (req, res) => {
    try {
        const check_row = await Campaign.findOne({ where: { id: req.body.id, user_id: req._user_id } })

        if (check_row) {
            await Campaign.update(
                req.body,
                {
                    where: { id: req.body.id, user_id: req._user_id }
                }
            )

            res.send({ status: 200, message: "Successfully Updated." })
        } else {
            res.send({ status: 404, message: "Something Went Wrong!" })
        }

    } catch (error) {
        res.send({ status: 404, message: error.message })
    }
}


createTrimVideo = async (req, res) => {
    try {
        const { url, start_point, end_point, camp_id } = req.body;


        // console.log(url)

        const frame = await trimVideo(url, start_point, end_point, `/${req._user_id}/campaigns/${camp_id}/`)

        console.log(frame)
        res.send(frame)

    } catch (error) {
        console.log(error)
    }
}


gifCreation = async (req, res) => {
    try {
        const { id, gif_data,gif_url} = req.body

        const data = await Campaign.findOne({
            where: {
                user_id: req._user_id,
                id,
            }
        })

        const { start, end } = JSON.parse(gif_data)

        // const gif = await createGifFromVideo(data.video_path, id, `/${req._user_id}/campaigns/${id}/gif/`, start, end,gif_url)
        // console.log(process.env.vidamaze_AWS_LAMDA_GIF,gif_url)

        const gif = await axios.post(process.env.vidamaze_AWS_LAMDA_GIF, {
            url: data.video_path,
            start,
            end,
            gif_url:gif_url,
            camp_id:id,
            upload_dir_path:`/${req._user_id}/campaigns/${id}/gif/`
        });

        // console.log("lamda response="+gif.data.url);

        if (!gif.data.url) console.log('Something went wrong')

        // console.log(gif)

        await Campaign
            .update(
                {
                    gif_url: gif.data.url
                },
                {
                    where: { id }
                }
            )

        res.send({ status: 200, gif: gif.data.url });

        // console.log(data.video_path,start,end)


    } catch (error) {
        console.log(error)
        res.send({ status: 500, message: error })
    }
}


/* 
    Clone Campaign Model
        1 Campaign Model
        2 Personalization
*/

cloneCampaign = async (req, res) => {
    try {
        const { camp_id, name } = req.body;

        const check_row = await Campaign.findOne({ where: { id: camp_id, user_id: req._user_id } })

        // console.log(check_row.poster);
        const {
            keywords,
            campaign_data,
            audio_path,
            assets_data,
            description,
            video_type,
            template_id,
        } = check_row

        var insertData = await Campaign.create({
            name, template_id, user_id: req._user_id,
            campaign_data, keywords, audio_path, assets_data, description, video_type
        })


        const data = await Personalize.findAll({ where: { campaigns_id: camp_id } })
        const landingData = await Landingpage.findOne({
            where: {
                camp_id,
            },
        });

        // console.log(landingData)


        if (landingData) {
            // console.log(landingData)
            const { content, draft_content, status, landing_page_id } = landingData;



            const a = await Landingpage.create({
                camp_id: insertData.id,
                draft_content: content ? content : draft_content,
                content: content ? content : draft_content,
                landing_page_id,
                status
            })


        }

        if (data.length > 0) {
            data.map(ele => {
                // console.log(ele)
                const { meta_type, meta_data } = ele

                Personalize.create({
                    meta_type,
                    campaigns_id: insertData.id,
                    meta_data
                });
            })
            res.send({ status: 200, camp_id: insertData.id, personalize_status: true })

        } else {
            res.send({ status: 200, camp_id: insertData.id, personalize_status: false })
        }




    } catch (error) {
        console.log(error)
    }
}

getUserVideoCredits = async (req, res) => {
    try {
        const data = await Campaign.findAll({
            where: {
                user_id: req._user_id,
                process_flow: 3,
            },
            // logging: console.log,
            attributes: [[sequelize.fn('sum', sequelize.col('video_duration')),'total_duration']]
        });

        if(!data) return res.send({status: 402, body: null});

        const have_credit = await User.findOne({
            where: {
                id: req._user_id,
            },
            logging: console.log,
            attributes: ['video_credits'],
        });

        // console.log(have_credit.dataValues.video_credits);

        if(data) return res.send({status: 200, body: {total_duration:data[0].dataValues.total_duration,total_credits:have_credit.dataValues.video_credits}});
        
    } catch (error) {
        res.send({status: 500, message: error.message});
    }
}

restartRender = async (req, res) => {
    
    try {

        if(req._user_id === 1){

            const { camp_id } = req.body;

            const check_row = await Campaign.findOne({ where: { id: camp_id } })

            if (check_row) {
                await Campaign.update(
                    { render_progress: 0, process_flow:2 },
                    {
                        where: { id: camp_id}
                    }
                )
                

                res.send({ status: 200, message: "Successfully Updated." })
            } else {
                res.send({ status: 404, message: "Something Went Wrong!" })
            }
        }else{
            res.send({ status: 404, message: "Something Went Wrong!" })
        }

    } catch (error) {
        res.send({ status: 404, message: error.message })
    }
}


module.exports =
{
    checkTestEmail,
    checkCampaign,
    insert,
    insertCampaign,
    fetchCampaign,
    uploadImage,
    uploadVideo,
    uploadAudio,
    fetchImages,
    fetchMedia,
    uploadURLImage,
    saveVideoData,
    fetchSingleCampaign,
    configureAudio,
    setrenderData,
    getCampaigns,
    getAllCampaigns,
    deleteCampaign,
    updateCampaign,
    createTrimVideo,
    cloneCampaign,
    gifCreation,
    getUserVideoCredits,
    restartRender
}